package main

import (
	"os"

	"github.com/hashicorp/go-hclog"
	tf "github.com/hashicorp/vault-plugin-secrets-terraform"
	"github.com/hashicorp/vault/api"
	"github.com/hashicorp/vault/sdk/plugin"
)

func main() {
	apiClientMeta := &api.PluginAPIClientMeta{}
	flags := apiClientMeta.FlagSet()
	logger := hclog.New(&hclog.LoggerOptions{})
	if err := flags.Parse(os.Args[1:]); err != nil {
		logger.Error("error parsing arguments", "error", err)
		os.Exit(1)
	}

	tlsConfig := apiClientMeta.GetTLSConfig()
	tlsProviderFunc := api.VaultPluginTLSProvider(tlsConfig)

	err := plugin.Serve(&plugin.ServeOpts{
		BackendFactoryFunc: tf.Factory,
		TLSProviderFunc:    tlsProviderFunc,
	})
	if err != nil {
		logger.Error("plugin shutting down", "error", err)
		os.Exit(1)
	}
}
